/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.document;

import org.apache.commons.collections.MapUtils;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.NullValueCheckStrategy;

import de.ozgcloud.document.common.attached_item.AttachedItem;

@Mapper(nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS)
interface DocumentMapper {

	@Mapping(target = "allFields", ignore = true)
	@Mapping(target = "unknownFields", ignore = true)
	@Mapping(target = "typeBytes", ignore = true)
	@Mapping(target = "nachrichtTextBytes", ignore = true)
	@Mapping(target = "nachrichtSubjectBytes", ignore = true)
	@Mapping(target = "mergeUnknownFields", ignore = true)
	@Mapping(target = "mergeFrom", ignore = true)
	@Mapping(target = "idBytes", ignore = true)
	@Mapping(target = "fileIdBytes", ignore = true)
	@Mapping(target = "clearOneof", ignore = true)
	@Mapping(target = "clearField", ignore = true)
	GrpcDocument toGrpcDocument(Document document);

	default Document fromAttachedItem(AttachedItem attachedItem) {
		return Document.builder()
				.id(attachedItem.getId())
				.type(MapUtils.getString(attachedItem.getItem(), Document.FIELD_DOCUMENT_TYPE))
				.fileId(MapUtils.getString(attachedItem.getItem(), Document.FIELD_DOCUMENT_FILE))
				.nachrichtText(MapUtils.getString(attachedItem.getItem(), Document.FIELD_NACHRICHT_TEXT))
				.nachrichtSubject(MapUtils.getString(attachedItem.getItem(), Document.FIELD_NACHRICHT_SUBJECT))
				.build();
	}
}
