package de.ozgcloud.document.bescheid;

import static de.ozgcloud.document.bescheid.Bescheid.*;

import java.time.LocalDate;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;

import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.mapstruct.CollectionMappingStrategy;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.NullValueCheckStrategy;
import org.mapstruct.NullValuePropertyMappingStrategy;
import org.mapstruct.ReportingPolicy;
import org.springframework.beans.factory.annotation.Autowired;

import de.ozgcloud.command.Command;
import de.ozgcloud.common.binaryfile.FileId;
import de.ozgcloud.common.datatype.StringBasedValue;
import de.ozgcloud.document.bescheid.Bescheid.SendBy;
import de.ozgcloud.document.bescheid.vorgang.VorgangId;
import de.ozgcloud.document.common.attached_item.AttachedItem;
import de.ozgcloud.document.common.attached_item.AttachedItemService;

@Mapper(uses = { SentInfoMapper.class }, //
		unmappedTargetPolicy = ReportingPolicy.WARN, //
		collectionMappingStrategy = CollectionMappingStrategy.ADDER_PREFERRED, //
		nullValuePropertyMappingStrategy = NullValuePropertyMappingStrategy.IGNORE, //
		nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS) //
public abstract class BescheidMapper {

	@Autowired
	private SentInfoMapper sentInfoMapper;

	@Mapping(target = "sentInfoOrBuilder", ignore = true)
	@Mapping(target = "defaultInstanceForType", ignore = true)
	@Mapping(target = "mergeFrom", ignore = true)
	@Mapping(target = "clearField", ignore = true)
	@Mapping(target = "clearOneof", ignore = true)
	@Mapping(target = "mergeSentInfo", ignore = true)
	@Mapping(target = "mergeUnknownFields", ignore = true)
	@Mapping(target = "bescheidDocumentBytes", ignore = true)
	@Mapping(target = "beschiedenAmBytes", ignore = true)
	@Mapping(target = "idBytes", ignore = true)
	@Mapping(target = "nachrichtSubjectBytes", ignore = true)
	@Mapping(target = "nachrichtTextBytes", ignore = true)
	@Mapping(target = "sendByBytes", ignore = true)
	@Mapping(target = "statusBytes", ignore = true)
	@Mapping(target = "unknownFields", ignore = true)
	@Mapping(target = "allFields", ignore = true)
	@Mapping(target = "beschiedenAm", source = "beschiedenAm", dateFormat = "yyyy-MM-dd")
	@Mapping(target = "nachrichtText", expression = "java(fromOptional(bescheid.getNachrichtText()))")
	@Mapping(target = "nachrichtSubject", expression = "java(fromOptional(bescheid.getNachrichtSubject()))")
	@Mapping(target = "attachmentsList", source = "attachments")
	public abstract GrpcBescheid toBescheid(Bescheid bescheid);

	String toString(FileId fileId) {
		return fileId.toString();
	}

	String toString(BescheidId bescheidId) {
		return bescheidId.toString();
	}

	String fromOptional(Optional<String> optionalString) {
		return optionalString.orElse(StringUtils.EMPTY);
	}

	@Mapping(target = "featuresOrBuilder", ignore = true)
	@Mapping(target = "defaultInstanceForType", ignore = true)
	@Mapping(target = "mergeFrom", ignore = true)
	@Mapping(target = "clearField", ignore = true)
	@Mapping(target = "clearOneof", ignore = true)
	@Mapping(target = "mergeFeatures", ignore = true)
	@Mapping(target = "mergeUnknownFields", ignore = true)
	@Mapping(target = "javaVersionBytes", ignore = true)
	@Mapping(target = "unknownFields", ignore = true)
	@Mapping(target = "versionBytes", ignore = true)
	@Mapping(target = "allFields", ignore = true)
	public abstract GrpcBescheidManagerConfigResponse fromBescheidManagerConfig(BescheidManagerConfig bescheidManagerConfig);

	@Mapping(target = "defaultInstanceForType", ignore = true)
	@Mapping(target = "bescheidOrBuilder", ignore = true)
	@Mapping(target = "bescheid", ignore = true)
	@Mapping(target = "mergeFrom", ignore = true)
	@Mapping(target = "clearField", ignore = true)
	@Mapping(target = "clearOneof", ignore = true)
	@Mapping(target = "mergeBescheid", ignore = true)
	@Mapping(target = "mergeUnknownFields", ignore = true)
	@Mapping(target = "unknownFields", ignore = true)
	@Mapping(target = "allFields", ignore = true)
	public abstract GrpcGetBescheidResponse toGetBescheidResponse(Bescheid bescheid);

	@Mapping(target = "defaultInstanceForType", ignore = true)
	@Mapping(target = "bescheidOrBuilder", ignore = true)
	@Mapping(target = "mergeFrom", ignore = true)
	@Mapping(target = "clearField", ignore = true)
	@Mapping(target = "clearOneof", ignore = true)
	@Mapping(target = "mergeBescheid", ignore = true)
	@Mapping(target = "mergeUnknownFields", ignore = true)
	@Mapping(target = "unknownFields", ignore = true)
	@Mapping(target = "allFields", ignore = true)
	public abstract GrpcGetBescheidDraftResponse toGetBescheidDraftResponse(Bescheid bescheid);

	Bescheid mapFromAttachedItem(AttachedItem item) {
		return buildBescheidBuilder(item.getItem())
				.id(BescheidId.from(item.getId()))
				.version(item.getVersion())
				.vorgangId(VorgangId.from(item.getVorgangId()))
				.build();
	}

	Bescheid mapFromCommand(Command command) {
		return buildBescheidBuilder(command.getBodyObject())
				.vorgangId(VorgangId.from(command.getVorgangId()))
				.build();
	}

	private Bescheid.BescheidBuilder buildBescheidBuilder(Map<String, Object> bescheidMap) {
		var bescheidBuilder = Bescheid.builder();
		Optional.ofNullable(MapUtils.getBoolean(bescheidMap, Bescheid.FIELD_BEWILLIGT)).ifPresent(bescheidBuilder::bewilligt);
		getBeschiedenAm(bescheidMap).ifPresent(bescheidBuilder::beschiedenAm);
		getStatus(bescheidMap).ifPresent(bescheidBuilder::status);
		getCreatedBy(bescheidMap).ifPresent(bescheidBuilder::createdBy);
		getBescheidDocument(bescheidMap).ifPresent(bescheidBuilder::bescheidDocument);
		bescheidBuilder.nachrichtSubject(getNachrichtSubject(bescheidMap));
		bescheidBuilder.nachrichtText(getNachrichtText(bescheidMap));
		getSendBy(bescheidMap).ifPresent(bescheidBuilder::sendBy);
		mapToAttachments(bescheidMap.get(Bescheid.FIELD_ATTACHMENTS)).ifPresent(bescheidBuilder::attachments);
		getSentInfo(bescheidMap).ifPresent(bescheidBuilder::sentInfo);
		return bescheidBuilder;
	}

	Optional<LocalDate> getBeschiedenAm(Map<String, Object> bescheidMap) {
		return getStringValue(bescheidMap, Bescheid.FIELD_BESCHIEDEN_AM).map(LocalDate::parse);
	}

	Optional<Bescheid.Status> getStatus(Map<String, Object> bescheidMap) {
		return getStringValue(bescheidMap, Bescheid.FIELD_STATUS).map(Bescheid.Status::valueOf);
	}

	Optional<UserId> getCreatedBy(Map<String, Object> bescheidMap) {
		return getStringValue(bescheidMap, Bescheid.FIELD_CREATED_BY).map(UserId::from);
	}

	Optional<SendBy> getSendBy(Map<String, Object> bescheidMap) {
		return getStringValue(bescheidMap, Bescheid.FIELD_SEND_BY).map(SendBy::valueOf);
	}

	Optional<FileId> getBescheidDocument(Map<String, Object> bescheidMap) {
		return getStringValue(bescheidMap, Bescheid.FIELD_BESCHEID_DOCUMENT).map(FileId::from);
	}

	private Optional<String> getNachrichtSubject(Map<String, Object> bescheidMap) {
		return getStringValue(bescheidMap, Bescheid.FIELD_NACHRICHT_SUBJECT);
	}

	private Optional<String> getNachrichtText(Map<String, Object> bescheidMap) {
		return getStringValue(bescheidMap, Bescheid.FIELD_NACHRICHT_TEXT);
	}

	private Optional<String> getStringValue(Map<String, Object> srcMap, String field) {
		return Optional.ofNullable(MapUtils.getString(srcMap, field)).map(StringUtils::trimToNull);
	}

	Optional<List<FileId>> mapToAttachments(Object attachments) {
		if (Objects.isNull(attachments) || StringUtils.isEmpty(String.valueOf(attachments))) {
			return Optional.empty();
		}
		if (attachments instanceof Collection<?> attachmentList) {
			return Optional.of(attachmentList.stream().map(String::valueOf).map(FileId::from).toList());
		}
		return Optional.of(Collections.singletonList(FileId.from((String) attachments)));
	}

	private Optional<SentInfo> getSentInfo(Map<String, Object> itemMap) {
		return Optional.ofNullable(MapUtils.getMap(itemMap, Bescheid.FIELD_SENT_INFO)).map(sentInfoMapper::toSentInfo);
	}

	@Mapping(target = "client", constant = BescheidCallContextAttachingInterceptor.BESCHEID_MANAGER_CLIENT_NAME)
	@Mapping(target = "itemName", constant = AttachedItemService.BESCHEID_ITEM_NAME)
	@Mapping(target = "item", expression = "java(mapToItem(bescheid))")
	public abstract AttachedItem mapFromBescheid(Bescheid bescheid);

	String mapToString(StringBasedValue value) {
		return value.toString();
	}

	public Map<String, Object> mapToItem(Bescheid bescheid) {
		var result = new HashMap<String, Object>();
		result.put(Bescheid.FIELD_BEWILLIGT, bescheid.isBewilligt());
		mapBeschiedenAm(bescheid).ifPresent(beschiedenAm -> result.put(Bescheid.FIELD_BESCHIEDEN_AM, beschiedenAm));
		mapStatus(bescheid).ifPresent(status -> result.put(Bescheid.FIELD_STATUS, status));
		mapBescheidDocument(bescheid).ifPresent(bescheidDocumentId -> result.put(Bescheid.FIELD_BESCHEID_DOCUMENT, bescheidDocumentId));
		mapSendBy(bescheid).ifPresent(sendBy -> result.put(Bescheid.FIELD_SEND_BY, sendBy));
		bescheid.getNachrichtSubject().ifPresent(subject -> result.put(Bescheid.FIELD_NACHRICHT_SUBJECT, subject));
		bescheid.getNachrichtText().ifPresent(text -> result.put(Bescheid.FIELD_NACHRICHT_TEXT, text));
		mapSentInfo(bescheid).ifPresent(sentInfo -> result.put(FIELD_SENT_INFO, sentInfo));
		mapAttachmentIds(bescheid.getAttachments()).ifPresent(attachmentIds -> result.put(FIELD_ATTACHMENTS, attachmentIds));
		return Collections.unmodifiableMap(result);
	}

	private Optional<String> mapBeschiedenAm(Bescheid bescheid) {
		return Optional.ofNullable(bescheid.getBeschiedenAm()).map(LocalDate::toString);
	}

	private Optional<String> mapStatus(Bescheid bescheid) {
		return Optional.ofNullable(bescheid.getStatus()).map(Bescheid.Status::name);
	}

	private Optional<String> mapBescheidDocument(Bescheid bescheid) {
		return Optional.ofNullable(bescheid.getBescheidDocument()).map(FileId::toString);
	}

	private Optional<String> mapSendBy(Bescheid bescheid) {
		return Optional.ofNullable(bescheid.getSendBy()).map(Bescheid.SendBy::name);
	}

	public Optional<Map<String, Object>> mapSentInfo(Bescheid bescheid) {
		return Optional.ofNullable(bescheid.getSentInfo()).map(sentInfoMapper::toMap);
	}

	Optional<List<String>> mapAttachmentIds(List<FileId> attachments) {
		return attachments.isEmpty() ? Optional.empty() : Optional.of(attachments.stream().map(FileId::toString).toList());
	}

}