package de.ozgcloud.document.bescheid.common.freemarker;

import java.io.IOException;
import java.io.StringWriter;

import org.springframework.stereotype.Component;

import de.ozgcloud.common.errorhandling.TechnicalException;
import freemarker.template.Configuration;
import freemarker.template.Template;
import freemarker.template.TemplateException;
import lombok.RequiredArgsConstructor;

@Component
@RequiredArgsConstructor
public class TemplateHandler {

	private final Configuration freemarkerConfig;

	public String getRawTemplate(String templateName) {
		return getTemplate(templateName).toString();
	}

	public String fillTemplate(String templateName, Object dataModel) {
		try {
			var template = getTemplate(templateName);
			var stringWriter = new StringWriter();
			template.process(dataModel, stringWriter);
			return stringWriter.toString();

		} catch (IOException | TemplateException e) {
			throw new TechnicalException("Error filling template", e);
		}
	}

	private Template getTemplate(String templateName) {
		try {
			return freemarkerConfig.getTemplate(templateName);
		} catch (IOException e) {
			throw new TechnicalException("Error loading mail template", e);
		}
	}
}
