package de.ozgcloud.document.bescheid.smartdocuments;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.client.reactive.ReactorClientHttpConnector;
import org.springframework.web.reactive.function.client.ExchangeFilterFunctions;
import org.springframework.web.reactive.function.client.WebClient;

import reactor.netty.http.client.HttpClient;
import reactor.netty.transport.ProxyProvider;

@Configuration
@ConditionalOnProperty("ozgcloud.bescheid.smart-documents.url")
class SmartDocumentsConfiguration {

	@Autowired
	private SmartDocumentsProperties properties;

	@Bean("smartDocuments")
	WebClient smartDocumentsWebClient() {
		ReactorClientHttpConnector connector = new ReactorClientHttpConnector(buildHttpClient());

		return WebClient.builder()
				.baseUrl(properties.getUrl())
				.filter(ExchangeFilterFunctions.basicAuthentication(properties.getBasicAuth().getUsername(), properties.getBasicAuth().getPassword()))
				.clientConnector(connector)
				.build();
	}

	private HttpClient buildHttpClient() {
		if (properties.getProxy() != null) {
			return createProxyHttpClient();
		} else {
			return createNoProxyHttpClient();
		}
	}

	private HttpClient createNoProxyHttpClient() {
		return HttpClient.create();
	}

	private HttpClient createProxyHttpClient() {
		return HttpClient.create()
				.proxy(proxy -> proxy.type(ProxyProvider.Proxy.HTTP)
						.host(properties.getProxy().getHost())
						.port(properties.getProxy().getPort())
						.username(properties.getProxy().getUsername())
						.password(username -> properties.getProxy().getPassword()));
	}

}
