package de.ozgcloud.document.bescheid.smartdocuments;

import jakarta.validation.Valid;
import jakarta.validation.constraints.NotNull;

import org.apache.logging.log4j.core.config.plugins.validation.constraints.NotBlank;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Configuration;
import org.springframework.validation.annotation.Validated;

import lombok.Getter;
import lombok.Setter;

@ConditionalOnProperty("ozgcloud.bescheid.smart-documents.url")
@Validated
@Getter
@Setter
@Configuration
@ConfigurationProperties("ozgcloud.bescheid.smart-documents")
public class SmartDocumentsProperties {

	/**
	 * Location of the Smart Documents Server
	 */
	@NotBlank
	private String url;

	/**
	 * Credential for basic auth to the Smart Documents Server
	 */
	@NotNull
	@Valid
	private UsernamePassword basicAuth;

	/**
	 * Smart Documents Template Group
	 */
	@NotBlank
	private String templateGroup;
	/**
	 * Smart Documents Template
	 */
	@NotBlank
	private String template;

	/**
	 * Proxy Configuration if required.
	 */
	private ProxyConfiguration proxy;

	@Getter
	@Setter
	static class UsernamePassword {
		/**
		 * Username for authentication
		 */
		@NotBlank
		private String username;
		/**
		 * Password for authentication
		 */
		@NotBlank
		private String password;
	}

	@Getter
	@Setter
	static class ProxyConfiguration {
		/**
		 * Host Address of the proxy server with protocol
		 */
		@NotBlank
		private String host;
		/**
		 * Port Number of the proxy server
		 */
		private int port = 8080;
		/**
		 * Username for Authentication for the proxy server
		 */
		private String username;
		/**
		 * Username for Authentication for the proxy server
		 */
		private String password;
	}
}
