package de.ozgcloud.document.bescheid.vorgang;

import java.util.List;
import java.util.Map;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;

import lombok.Builder;
import lombok.Getter;
import lombok.Singular;
import lombok.ToString;

@Builder
@Getter
@ToString
public class Vorgang {

	@JsonIgnore
	private VorgangId id;
	private long version;

	private String vorgangName;

	private String vorgangNummer;
	private String aktenzeichen;

	@JsonIgnore
	@ToString.Exclude
	private ServiceKonto serviceKonto;
	@ToString.Exclude
	private Eingang eingang;

	@Builder
	@Getter
	public static class ServiceKonto {

		public static final String FIELD_SERVICEKONTO_TYPE = "serviceKontoType";
		public static final String FIELD_POSTFACH_ADDRESS = "postfachAddress";

		private String type;
		@Singular
		private List<PostfachAddress> postfachAddresses;
	}

	@Builder
	@Getter
	public static class PostfachAddress {

		public static final String FIELD_VERSION = "version";
		public static final String FIELD_TYPE = "type";
		public static final String FIELD_IDENTIFIER = "identifier";

		private String version;
		private int type;
		private Map<String, Object> identifier;
	}

	@Builder
	@Getter
	static class Eingang {
		private Antragsteller antragsteller;
		private ZustaendigeStelle zustaendigeStelle;

		@Singular("formData")
		@JsonSerialize(using = FormDataEntryListSerializer.class)
		private List<FormDataEntry> formData;
	}

	@Builder
	@Getter
	@JsonInclude(Include.NON_EMPTY)
	static class Antragsteller {
		private String anrede;
		private String nachname;
		private String vorname;
		private String geburtsdatum;
		private String geburtsort;
		private String geburtsname;
		private String email;
		private String telefon;
		private String strasse;
		private String hausnummer;
		private String plz;
		private String ort;
	}

	@Builder
	@Getter
	static class ZustaendigeStelle {
		private String organisationseinheitenId;
		private String email;
	}

}
