/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.document.bescheid.common.callcontext;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.Optional;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.apilib.user.OzgCloudUserId;
import de.ozgcloud.document.bescheid.BescheidCallContextAttachingInterceptor;

class BescheidManagerCallContextProviderTest {

	@Spy
	@InjectMocks
	private BescheidManagerCallContextProvider provider;

	@Mock
	private CurrentUserService currentUserService;
	@Mock
	private CallContextUser contextUser;
	@Mock
	private OzgCloudUserId userId;

	@Nested
	class TestProvideContext {

		@BeforeEach
		void init() {
			doReturn(Optional.of(userId)).when(provider).getUserId();
		}

		@Test
		void shouldCallGetUserId() {
			provider.provideContext();

			verify(provider).getUserId();
		}

		@Test
		void shouldSetUserId() {
			var result = provider.provideContext();

			assertThat(result.getUserId()).isSameAs(userId);
		}

		@Test
		void shouldSetClientName() {
			var result = provider.provideContext();

			assertThat(result.getClientName()).isEqualTo(BescheidCallContextAttachingInterceptor.BESCHEID_MANAGER_CLIENT_NAME);
		}
	}

	@Nested
	class TestGetUserId {

		private static final String USER_ID = "userId";

		@BeforeEach
		void init() {
			when(currentUserService.getUser()).thenReturn(contextUser);
		}

		@Test
		void shouldReturnUserId() {
			when(contextUser.getUserId()).thenReturn(Optional.of(USER_ID));

			var result = provider.getUserId();

			assertThat(result).isPresent().get().hasToString(USER_ID);
		}

	}
}